@@ -2,18 +2,23 @@
 # -*- coding: utf-8 -*-
 
 import logging
+import warnings
 from typing import Any, Dict, List, Optional, Tuple, Type
 
 import dask.array as da
 import numpy as np
-from distributed import Client, LocalCluster
 
-from . import dask_utils, transforms, types
+from . import transforms, types
 from .constants import Dimensions
-from .exceptions import (InvalidDimensionOrderingError,
-                         UnsupportedFileFormatError)
-from .readers import (ArrayLikeReader, CziReader, DefaultReader, OmeTiffReader,
-                      TiffReader)
+from .exceptions import InvalidDimensionOrderingError, UnsupportedFileFormatError
+from .readers import (
+    ArrayLikeReader,
+    CziReader,
+    DefaultReader,
+    LifReader,
+    OmeTiffReader,
+    TiffReader,
+)
 from .readers.reader import Reader
 
 ###############################################################################
@@ -23,8 +28,17 @@ log = logging.getLogger(__name__)
 ###############################################################################
 
 # The order of the readers in this list is important.
-# Example: if TiffReader was placed before OmeTiffReader, we would never hit the OmeTiffReader.
-SUPPORTED_READERS = [ArrayLikeReader, CziReader, OmeTiffReader, TiffReader, DefaultReader]
+# Example:
+# if TiffReader was placed before OmeTiffReader,
+# we would never hit the OmeTiffReader
+SUPPORTED_READERS = [
+    ArrayLikeReader,
+    CziReader,
+    LifReader,
+    OmeTiffReader,
+    TiffReader,
+    DefaultReader,
+]
 
 ###############################################################################
 
@@ -35,26 +49,30 @@ class AICSImage:
         data: types.ImageLike,
         known_dims: Optional[str] = None,
         dask_kwargs: Dict[str, Any] = {},
-        **kwargs
+        **kwargs,
     ):
         """
-        AICSImage takes microscopy image data types (files) of varying dimensions ("ZYX", "TCZYX", "CYX") and
-        puts them into a consistent 6D "STCZYX" ordered dask array. The data, metadata are lazy loaded and can be
-        accessed as needed. Note the dims are assumed to match "STCZYX" from right to left meaning if 4 dimensional
-        data is provided then the dimensions are assigned to be "CZYX", 2 dimensional would be "YX". This guessed
-        assignment is only for file types without dimension metadata (i.e. not .ome.tiff or .czi).
+        AICSImage takes microscopy image data types (files) of varying dimensions
+        ("ZYX", "TCZYX", "CYX") and puts them into a consistent 6D "STCZYX" ordered
+        dask array. The data, metadata are lazy loaded and can be accessed as needed.
+        Note the dims are assumed to match "STCZYX" from right to left meaning if
+        dimensional data is provided then the dimensions are assigned to be "CZYX", 2
+        dimensional would be "YX". This guessed assignment is only for file types
+        without dimension metadata (i.e. not .ome.tiff or .czi).
 
         Parameters
         ----------
         data: types.ImageLike
-            String with path to file, numpy.ndarray, or dask.array.Array with up to six dimensions.
+            String with path to file, numpy.ndarray, or dask.array.Array with up to six
+            dimensions.
         known_dims: Optional[str]
-            Optional string with the known dimension order. If None, the reader will attempt to parse dim order.
+            Optional string with the known dimension order. If None, the reader will
+            attempt to parse dim order.
         dask_kwargs: Dict[str, Any] = {}
             A dictionary of arguments to pass to a dask cluster and or client.
         kwargs: Dict[str, Any]
-            Extra keyword arguments that can be passed down to either the reader subclass or, if using the context
-            manager, the LocalCluster initialization.
+            Extra keyword arguments that can be passed down to either the reader
+            subclass or, if using the context manager, the LocalCluster initialization.
 
         Examples
         --------
@@ -63,7 +81,8 @@ class AICSImage:
         >>> img = AICSImage("my_file.tiff")
         ... zstack_t8 = img.get_image_data("ZYX", S=0, T=8, C=0)
 
-        Initialize an image, construct a delayed dask array for certain slices, then read the data.
+        Initialize an image, construct a delayed dask array for certain slices, then
+        read the data.
 
         >>> img = AICSImage("my_file.czi")
         ... zstack_t8 = img.get_image_dask_data("ZYX", S=0, T=8, C=0)
@@ -83,29 +102,38 @@ class AICSImage:
         >>> with AICSImage("filename.ome.tiff") as img:
         ...     data = img.get_image_data("ZYX", S=0, T=0, C=0)
 
-        Create a local dask cluster with arguments for the duration of the context manager.
+        Create a local dask cluster with arguments for the duration of the context
+        manager.
 
         >>> with AICSImage("filename.ome.tiff", dask_kwargs={"nworkers": 4}) as img:
         ...     data = img.get_image_data("ZYX", S=0, T=0, C=0)
 
         Connect to a specific dask cluster for the duration of the context manager.
 
-        >>> with AICSImage("filename.ome.tiff", dask_kwargs={"address": "tcp://localhost:1234"}) as img:
+        >>> with AICSImage(
+        ...     "filename.ome.tiff",
+        ...     dask_kwargs={"address": "tcp://localhost:1234"},
+        ...     ) as img:
         ...     data = img.get_image_data("ZYX", S=0, T=0, C=0)
-        ```
 
         Notes
         -----
-        Constructor for AICSImage class intended for providing a unified interface for dealing with
-        microscopy images. To extend support to a new reader simply add a new reader child class of
-        Reader ([readers/reader.py]) and add the class to SUPPORTED_READERS variable.
+        When using the AICSImage context manager, the processing machine or container
+        must have networking capabilities enabled to function properly.
+
+        Constructor for AICSImage class intended for providing a unified interface for
+        dealing with microscopy images. To extend support to a new reader simply add a
+        new reader child class of Reader ([readers/reader.py]) and add the class to
+        SUPPORTED_READERS variable.
         """
         # Check known dims
         if known_dims is not None:
             if not all([d in Dimensions.DefaultOrder for d in known_dims]):
                 raise InvalidDimensionOrderingError(
-                    f"The provided dimension string to the 'known_dims' argument includes dimensions that AICSImage "
-                    f"does not support. Received: '{known_dims}'. Supported dimensions: {Dimensions.DefaultOrderList}."
+                    f"The provided dimension string to the 'known_dims' argument "
+                    f"includes dimensions that AICSImage does not support. "
+                    f"Received: '{known_dims}'. "
+                    f"Supported dimensions: {Dimensions.DefaultOrderList}."
                 )
 
         # Hold onto known dims until data is requested
@@ -130,7 +158,8 @@ class AICSImage:
     @staticmethod
     def determine_reader(data: types.ImageLike) -> Type[Reader]:
         """
-        Cheaply check to see if a given file is a recognized type and return the appropriate reader for the file.
+        Cheaply check to see if a given file is a recognized type and return the
+        appropriate reader for the file.
         """
         # Iterate through the ordered supported readers to find the right one
         for reader_class in SUPPORTED_READERS:
@@ -163,7 +192,11 @@ class AICSImage:
         Return the entire image as a numpy array with dimension ordering "STCZYX".
         """
         if self._data is None:
-            self._data = self.dask_data.compute()
+            self._data = transforms.reshape_data(
+                data=self.reader.data,
+                given_dims=self._known_dims or self.reader.dims,
+                return_dims=self.dims,
+            )
 
         return self._data
 
@@ -172,7 +205,8 @@ class AICSImage:
         Parameters
         ----------
         dims: str
-            A string containing a list of dimensions being requested. The default is to return the six standard dims.
+            A string containing a list of dimensions being requested. The default is to
+            return the six standard dims.
 
         Returns
         -------
@@ -269,8 +303,8 @@ class AICSImage:
         Returns
         -------
         metadata: Any
-            The Metadata from the Czi, or Ome.Tiff file, or other base class type with metadata.
-            For pure image files an empty string or None is returned.
+            The Metadata from the Czi, or Ome.Tiff file, or other base class type with
+            metadata. For pure image files an empty string or None is returned.
         """
         # The reader can implement read optimization or not.
         return self.reader.metadata
@@ -289,7 +323,9 @@ class AICSImage:
         """
         return self._reader
 
-    def get_image_dask_data(self, out_orientation: Optional[str] = None, **kwargs) -> da.core.Array:
+    def get_image_dask_data(
+        self, out_orientation: Optional[str] = None, **kwargs
+    ) -> da.core.Array:
         """
         Get specific dimension image data out of an image as a dask array.
 
@@ -300,19 +336,60 @@ class AICSImage:
             Default: The current image dimensions. i.e. `self.dims`
 
         kwargs:
-            C=1: specifies Channel 1
-            T=3: specifies the fourth index in T
-            D=n: D is Dimension letter and n is the index desired D should not be present in the out_orientation
+            * C=1: specifies Channel 1
+            * T=3: specifies the fourth index in T
+            * D=n: D is Dimension letter and n is the index desired. D should not be
+              present in the out_orientation.
+            * D=[a, b, c]: D is Dimension letter and a, b, c is the list of indicies
+              desired. D should be present in the out_orientation.
+            * D=(a, b, c): D is Dimension letter and a, b, c is the tuple of indicies
+              desired. D should be present in the out_orientation.
+            * D=range(...): D is Dimension letter and range is the standard Python
+              range function. D should be present in the out_orientation.
+            * D=slice(...): D is Dimension letter and slice is the standard Python
+              slice function. D should be present in the out_orientation.
 
         Returns
         -------
         data: dask array
-            The read data with the dimension ordering that was specified with out_orientation.
+            The read data with the dimension ordering that was specified with
+            out_orientation.
 
-        Note: If a requested dimension is not present in the data the dimension is added with
-        a depth of 1.
+        Examples
+        --------
+        Specific index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... c1 = img.get_image_dask_data("ZYX", C=1)
+
+        List of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_and_second = img.get_image_dask_data("CZYX", C=[0, 1])
+
+        Tuple of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_and_last = img.get_image_dask_data("CZYX", C=(0, -1))
+
+        Range of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_three = img.get_image_dask_data("CZYX", C=range(3))
+
+        Slice selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... every_other = img.get_image_dask_data("CZYX", C=slice(0, -1, 2))
+
+        Notes
+        -----
+        * If a requested dimension is not present in the data the dimension is
+          added with a depth of 1.
+
+        See `aicsimageio.transforms.reshape_data` for more details.
         """
-        # If no out orientation, simply return current data as numpy array
+        # If no out orientation, simply return current data as dask array
         if out_orientation is None:
             return self.dask_data
 
@@ -324,7 +401,9 @@ class AICSImage:
             **kwargs,
         )
 
-    def get_image_data(self, out_orientation: Optional[str] = None, **kwargs) -> np.ndarray:
+    def get_image_data(
+        self, out_orientation: Optional[str] = None, **kwargs
+    ) -> np.ndarray:
         """
         Get specific dimension image data out of an image as a numpy array.
 
@@ -335,19 +414,62 @@ class AICSImage:
             Default: The current image dimensions. i.e. `self.dims`
 
         kwargs:
-            C=1: specifies Channel 1
-            T=3: specifies the fourth index in T
-            D=n: D is Dimension letter and n is the index desired D should not be present in the out_orientation
+            * C=1: specifies Channel 1
+            * T=3: specifies the fourth index in T
+            * D=n: D is Dimension letter and n is the index desired. D should not be
+              present in the out_orientation.
+            * D=[a, b, c]: D is Dimension letter and a, b, c is the list of indicies
+              desired. D should be present in the out_orientation.
+            * D=(a, b, c): D is Dimension letter and a, b, c is the tuple of indicies
+              desired. D should be present in the out_orientation.
+            * D=range(...): D is Dimension letter and range is the standard Python
+              range function. D should be present in the out_orientation.
+            * D=slice(...): D is Dimension letter and slice is the standard Python
+              slice function. D should be present in the out_orientation.
+
+        Examples
+        --------
+        Specific index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... c1 = img.get_image_data("ZYX", C=1)
+
+        List of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_and_second = img.get_image_data("CZYX", C=[0, 1])
+
+        Tuple of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_and_last = img.get_image_data("CZYX", C=(0, -1))
+
+        Range of index selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... first_three = img.get_image_data("CZYX", C=range(3))
+
+        Slice selection
+
+        >>> img = AICSImage("s_1_t_1_c_10_z_20.ome.tiff")
+        ... every_other = img.get_image_data("CZYX", C=slice(0, -1, 2))
 
         Returns
         -------
         data: np.ndarray
-            The read data with the dimension ordering that was specified with out_orientation.
+            The read data with the dimension ordering that was specified with
+            out_orientation.
 
-        Note: If a requested dimension is not present in the data the dimension is added with
-        a depth of 1.
+        Notes
+        -----
+        * If a requested dimension is not present in the data the dimension is
+          added with a depth of 1.
+
+        See `aicsimageio.transforms.reshape_data` for more details.
         """
-        return self.get_image_dask_data(out_orientation=out_orientation, **kwargs).compute()
+        return self.get_image_dask_data(
+            out_orientation=out_orientation, **kwargs
+        ).compute()
 
     def view_napari(self, rgb: bool = False, **kwargs):
         """
@@ -364,7 +486,8 @@ class AICSImage:
         try:
             import napari
 
-            # Construct getitem operations tuple to select down the data in the filled dimensions
+            # Construct getitem operations tuple to select down the data
+            # in the filled dimensions
             ops = []
             selected_dims = []
             for dim in self.dims:
@@ -391,9 +514,7 @@ class AICSImage:
                 # Swap channel to last dimension
                 new_dims = f"{dims.replace(Dimensions.Channel, '')}{Dimensions.Channel}"
                 data = transforms.transpose_to_dims(
-                    data=data,
-                    given_dims=dims,
-                    return_dims=new_dims
+                    data=data, given_dims=dims, return_dims=new_dims
                 )
 
                 # Run napari
@@ -405,13 +526,17 @@ class AICSImage:
                         title=title,
                         axis_labels=dims.replace(Dimensions.Channel, ""),
                         rgb=rgb,
-                        **kwargs
+                        **kwargs,
                     )
 
             # Handle all other images besides RGB not requested
             else:
                 # Channel axis
-                c_axis = dims.index(Dimensions.Channel) if Dimensions.Channel in dims else None
+                c_axis = (
+                    dims.index(Dimensions.Channel)
+                    if Dimensions.Channel in dims
+                    else None
+                )
 
                 # Set visible based on number of channels
                 if c_axis is not None:
@@ -423,7 +548,11 @@ class AICSImage:
                     visible = True
 
                 # Drop channel from dims string
-                dims = dims.replace(Dimensions.Channel, "") if Dimensions.Channel in dims else dims
+                dims = (
+                    dims.replace(Dimensions.Channel, "")
+                    if Dimensions.Channel in dims
+                    else dims
+                )
 
                 # Run napari
                 with napari.gui_qt():
@@ -435,13 +564,14 @@ class AICSImage:
                         axis_labels=dims,
                         title=title,
                         visible=visible,
-                        **kwargs
+                        **kwargs,
                     )
 
         except ModuleNotFoundError:
             raise ModuleNotFoundError(
-                f"'napari' has not been installed. To use this function install napari with either: "
-                f"'pip install napari' or 'pip install aicsimageio[interactive]'"
+                "'napari' has not been installed. To use this function install napari "
+                "with either: pip install napari' or "
+                "'pip install aicsimageio[interactive]'"
             )
 
     def get_channel_names(self, scene: int = 0) -> List[str]:
@@ -458,11 +588,17 @@ class AICSImage:
         channels_names: List[str]
             List of strings representing the channel names.
         """
-        try:
-            return self.reader.get_channel_names(scene)
-        except AttributeError:
+        # Get channel names from reader
+        channel_names = self.reader.get_channel_names(scene)
+
+        # Unlike the readers, AICSImage objects always have a channel dimension
+        # In the case the base reader returns None, return a list of "0"
+        if channel_names is None:
             return [str(i) for i in range(self.size_c)]
 
+        # Return the read channel names
+        return channel_names
+
     def get_physical_pixel_size(self, scene: int = 0) -> Tuple[float]:
         """
         Attempts to retrieve physical pixel size for the specified scene.
@@ -484,14 +620,14 @@ class AICSImage:
         return f"<AICSImage [{type(self.reader).__name__}]>"
 
     @property
-    def cluster(self) -> Optional[LocalCluster]:
+    def cluster(self) -> Optional["distributed.LocalCluster"]:  # noqa: F821
         """
         If this object created a local Dask cluster, return it.
         """
         return self._cluster
 
     @property
-    def client(self) -> Optional[Client]:
+    def client(self) -> Optional["distributed.Client"]:  # noqa: F821
         """
         If connected to a Dask cluster, return the connected Client.
         """
@@ -502,15 +638,33 @@ class AICSImage:
         Close the connection to the Dask distributed Client.
         If this object created a LocalCluster, close it down as well.
         """
-        self._cluster, self._client = dask_utils.shutdown_cluster_and_client(self.cluster, self.client)
+        from . import dask_utils
+
+        self._cluster, self._client = dask_utils.shutdown_cluster_and_client(
+            self.cluster, self.client
+        )
 
     def __enter__(self):
         """
         If provided an address, create a Dask Client connection.
         If not provided an address, create a LocalCluster and Client connection.
-        If not provided an address, other Dask kwargs are accepted and passed down to the LocalCluster object.
+        If not provided an address, other Dask kwargs are accepted and passed down to
+        the LocalCluster object.
         """
-        self._cluster, self._client = dask_utils.spawn_cluster_and_client(**self._dask_kwargs)
+        # Warn of future changes to API
+        warnings.warn(
+            "In aicsimageio>=4.*, the AICSImage context manager will no longer "
+            "construct and manage a distributed local dask cluster for you. If this "
+            "functionality is desired for your work, please switch to explictly "
+            "calling the `aicsimageio.dask_utils.cluster_and_client` context manager.",
+            FutureWarning,
+        )
+
+        from . import dask_utils
+
+        self._cluster, self._client = dask_utils.spawn_cluster_and_client(
+            **self._dask_kwargs
+        )
 
         return self
 
